

# This script aims to create mask for the regions computing SPEI at EEA river subbasin level

library(sf)
library(readxl)
library(dplyr)
library(easyNCDF)

path.downloaded_data <- 'cds/'
path.intermediate_data <- 'folder/'

# Set the file path to the shapefile with EEA subbasins
mask.file <- paste0(path.intermediate_data, '/exposure_data/WEI/SB_2022_v16/SB_2022_v16_3.shp')
shp <- st_read(mask.file)

# Define the target CRS
target_crs <- "+init=epsg:4326"
transformed_shapefile <- st_transform(shp, crs = target_crs)
shp <- transformed_shapefile

# Check for invalid polygons
invalid_indices <- st_is_valid(shp, reason = FALSE)
invalid_indices <- which(invalid_indices == FALSE)
invalid_polygons <- shp[invalid_indices, ]
fixed_polygons <- st_make_valid(invalid_polygons)
shp[invalid_indices, ] <- fixed_polygons
is_valid <- st_is_valid(shp)


# Read CSV and extract location of the ones that is needed
# WEI+ 1990-2015

data_1990 <- read_excel(paste0(path.intermediate_data,"exposure_data/WEI_river_sub-basin_1990-2015.xls"))
column <- data_1990[['Entity']]
region_code_1990 <- unique(column)
n_reg_1990 <- length(region_code_1990)

region_code_1990_sorted <- sort(region_code_1990)

# WEI+ 2016-2019

data_2016 <- read_excel(paste0(path.intermediate_data,'exposure_data/WEI_river_sub-basin_2016-2019.xlsx', sheet = 'DATASET_for_Mapping'))
column <- data_2016[['Entity']]
region_code_2016 <- unique(column)
n_reg_2016 <- length(region_code_2016)

# Combined code
combined_region_codes <- union(region_code_1990, region_code_2016)

# Cropped common regions 
shp_0 <- shp[shp$SB_REF_V16 %in% combined_region_codes, ]
shp_0 <- shp_0 %>% arrange(SB_REF_V16)

region_code <- shp_0$SB_REF_V16
shp <- shp_0

# Set longitude and latitude values

ref.file <- paste0(path.downloaded_data,'era5land/monthly_mean/tasmax_f24h/tasmax_202101.nc')
lat <- NcToArray(ref.file, vars_to_read = 'lat')
lon <- NcToArray(ref.file, vars_to_read = 'lon')
lat <- lat[1, ]
lon <- lon[1, ]
nlon <- length(lon)

trace_longitudes <- array(NA,dim=nlon)
trace_longitudes[1:(nlon/2)] <- lon[(nlon/2+1):nlon]-360
trace_longitudes[(nlon/2+1):nlon] <- lon[1:(nlon/2)]

lon <- trace_longitudes

# European Domain
lat_min <- 25
lat_max <- 75
lon_min <- -15
lon_max <- 44

lat_values <- lat[which(lat==lat_min):which(lat==lat_max)]
lon_values <- lon[which(lon==lon_min):which(lon==lon_max)]

n_countries <- length(region_code)  # Number of countries
n_lon <- length(lon_values)     # Number of longitude values
n_lat <- length(lat_values)     # Number of latitude values

# Create a grid of points using longitude and latitude values
grid_points <- expand.grid(lon = lon_values, lat = lat_values)
grid_points_sf <- st_as_sf(grid_points, coords = c("lon", "lat"), crs = target_crs)

# Create an empty mask array
mask <- array(0, dim = c(n_countries, length(lon_values), length(lat_values)))

# Iterate over each country
for (i in 1:n_countries) {
  
  print(i)
  
  # Get the polygon of the current country
  polygon <- shp[i, ]
  
  # is_valid <- st_is_valid(polygon)
  
  # Create a spatial index for the polygon
  polygon_index <- st_intersects(polygon, grid_points_sf)
  
  # Check if each grid point falls within the country polygon
  mask[i,,] <- apply(polygon_index, 2, function(x) as.integer(any(x)))
}


mask <- aperm(mask, c(1,3,2))
mask <- mask[,length(lat_values):1,]

mask_europe <- list()
mask_europe$mask <- mask
mask_europe$region_code <- region_code

saveRDS(mask_europe, file=paste(path.intermediate_data, 'rdata/EEA_subbasin_mask_europe_era5land_COMBINED_REGIONS_1990-2019.Rds', sep=""))