
# This script organizes the Quaterly WEI+ data at subbasin level that will be used for the analysis
# The excel sheets obtained from EEA are transformed to multi-dimensional array for analysis

library(readxl) # Reading spreadsheet
library(sf)
library(ggplot2)
library(ggspatial)
library(dplyr)
library(colorspace)
library(tidyr)
library(s2dv)

path.intermediate_data <- 'folder/'
path.final_data <- 'data_out/'

# Define ranges for drought and water stress conditon:

ranges_spei_all <- c(-1.3) # 'severe'
ranges_wei_all <- c(20, 40) # 'Water stress' 'High water stress'


# WEI+ 1990-2015

data_1990_q1 <- read_excel(path.intermediate_data, "exposure_data/original_wei_1990_2015/SB_WEI_MAA_Q1_v4.xlsx")
data_1990_q2 <- read_excel(path.intermediate_data, "exposure_data/original_wei_1990_2015/SB_WEI_MAA_Q2_v4.xlsx")
data_1990_q3 <- read_excel(path.intermediate_data, "exposure_data/original_wei_1990_2015/SB_WEI_MAA_Q3_v4.xlsx")
data_1990_q4 <- read_excel(path.intermediate_data, "exposure_data/original_wei_1990_2015/SB_WEI_MAA_Q4_v4.xlsx")

data_1990_q1 <- arrange(data_1990_q1, Entity)
data_1990_q2 <- arrange(data_1990_q2, Entity)
data_1990_q3 <- arrange(data_1990_q3, Entity)
data_1990_q4 <- arrange(data_1990_q4, Entity)

data_1990 <- rbind(data_1990_q1, data_1990_q2, data_1990_q3, data_1990_q4)
data_1990 <- data_1990[, c("Entity", "Year", "Season", "Water exploitation index plus", "Population")]
colnames(data_1990) <- c("Entity", "Year", "Season", "WEI","Population")

column <- data_1990[['Entity']]
region_code_1990 <- unique(column)
n_reg_1990 <- length(region_code_1990)

region_code_1990_sorted <- sort(region_code_1990)

# WEI+ 2016-2019

data_2016 <- read_excel(path.intermediate_data, 'exposure_data/WEI_river_sub-basin_2016-2019.xlsx', sheet = 'DATASET_for_Mapping')
data_2016 <- data_2016[, c("Entity", "Year", "Season", "WEI", "Area km2")]
colnames(data_2016) <- c("Entity", "Year", "Season", "WEI","Area_km2")

column <- data_2016[['Entity']]
region_code_2016 <- unique(column)
n_reg_2016 <- length(region_code_2016)

# Common codes, considered for LANCET report. Italy and UK removed - Only 292 river subbasin considered

common_codes <- readRDS(file=paste(path.intermediate_data, 'rdata/exposure_data/common_codes_all_data_available_1990_2019_EEA_riverbasin.Rds', sep=""))
common_codes <- setdiff(common_codes, "WSB0000508") # I remove "WSB0000508" since it is creating problems. The shape file has four different polygons for the same code. See 315-320th position of Entity_spei
count <- length(common_codes)

# Find common regions
common_regions_1990 <- data_1990[data_1990$'Entity' %in% common_codes, ]
common_regions_2016 <- data_2016[data_2016$'Entity' %in% common_codes, ]

# Order the data based on the common codes, years and season
common_regions_1990 <- common_regions_1990 %>%  arrange(match(Entity, common_codes))
common_regions_2016 <- common_regions_2016 %>%  arrange(match(Entity, common_codes))

# Creating population array for each Entity and year. For the years 2016-2019, 2015 population data is copied

population_Q3 <- common_regions_1990 %>%
  filter(Season == "Q3") %>%
  select(Entity, Year, Population)

# Filter the data for 2015
year_2015 <- population_Q3 %>%
  filter(Year == 2015)

# Create a new dataframe for the years 2016 to 2019 and populate them with the values from 2015
additional_years <- data.frame(
  Entity = unique(population_Q3$Entity),  # List of unique entities
  Year = rep(2016:2019, each = n_distinct(population_Q3$Entity)),
  Population = rep(year_2015$Population, times = n_distinct(population_Q3$Entity) * 4)
)

# Combine the original data with the additional years
updated_population_data <- bind_rows(population_Q3, additional_years) %>%
  arrange(Entity)

unique_population_data <- updated_population_data %>%
  group_by(Entity, Year) %>%
  summarize(Population = first(Population))

Entity_population <- unique(unique_population_data$Entity)
n_reg <- length(Entity_population)
n_year <- length(unique(unique_population_data$Year))

array_population <- array(unique_population_data$Population, dim=c(n_year,n_reg))
names(dim(array_population)) <- c('Year','Entity')

# Creating area array for each Entity and year. For the years 2016-2019, 2015 population data is copied

area_Q3 <- common_regions_2016 %>%
  filter(Year == 2016) %>%
  distinct(Entity, Year, Season, .keep_all = TRUE)  # Keep only the first occurrence for each Entity

unique_area_data <- area_Q3 %>%
  group_by(Entity, Year) %>%  # Group by Entity and Year
  summarize(Area_km2 = max(Area_km2))  # Calculate the maximum Area_km2

Entity_area <- unique_area_data$Entity
array_area <- InsertDim(unique_area_data$Area_km2,posdim = 1,lendim = n_year, name = "Year")
names(dim(array_area)) <- c('Year','Entity')

# write.table(common_codes, file = paste(path.final_data, 'considered_EEA_river_subbasin_Entity_for_analysis.csv', sep=""), sep = ",", col.names = NA, qmethod = "double")

# SPEI values
for (i in 12) {
  
  print(i)
  
  spei_index <- i
  
  # Extract only the relevant region
  target_months <- c(3,6,9,12)
  target_months_names <- c('March','June','September','December')
  
  seas_wei_names <- c('Q1','Q2','Q3','Q4')
  
  # SPEI: period 1990-2019
  mask_europe <- readRDS(file=paste(path.intermediate_data, 'rdata/EEA_subbasin_mask_europe_era5land_COMBINED_REGIONS_1990-2019.Rds', sep=""))
  Entity_spei <- sort(mask_europe$region_code)
  rm(mask_europe)
  
  regions <- 'EEA_subbasin'
  data_spei_obs <- readRDS(file=paste(path.intermediate_data, 'rdata/spei/data_pet_fao_spei_',i,'_',regions,'_COMBINED_REGIONS_1950-2019.Rds', sep=""))
  names(dim(data_spei_obs)) <- c('Month','Year','Entity')
  
  data_spei_obs <- data_spei_obs[target_months,40:69,] # 1990-2019

  # WEI array
  regions <-  'EEA_riverbasin'
  data_array <- readRDS(file=paste(path.intermediate_data, 'rdata/exposure_data/data_wei_',regions,'.Rds', sep=""))
  
  array_wei <- data_array$array
  wei_index_seasonal <- s2dv::Reorder(array_wei, order = c('Season','Year','Entity'))
  
  Entity_wei <- sort(data_array$Entity) 
  Year <- data_array$Year
  Season <- data_array$Season
  
  locations_spei <- which(Entity_spei %in% common_codes)
  locations_wei <- which(Entity_wei %in% common_codes)

  # Subselect only those regions where we have common data
  
  data_spei_obs <- data_spei_obs[,,locations_spei]
  wei_index_seasonal <- wei_index_seasonal[,,locations_wei]

  for (seas in 1:4) {
    
    tar_mon <- target_months[seas]
    target_months_name <- target_months_names[seas]
    
    seas_wei_name <- seas_wei_names[seas]
    
    data_spei_obs_mon <- data_spei_obs[seas,,]  # 11:20 for 2000-2009 / 21:30 for 2010-2019
    wei_index_seasonal_seas <- wei_index_seasonal[seas,,]  # 11:20 for 2000-2009 / 21:30 for 2010-2019
    
    data_spei_obs_mon_eu <- data_spei_obs_mon
    wei_index_seasonal_seas_eu <- wei_index_seasonal_seas
    
        range_spei <- c(-1.3)
        
        # How many severe drought events happened? 
        data_spei_obs_mon_event <- data_spei_obs_mon
        data_spei_obs_mon_event[which(data_spei_obs_mon_event > range_spei)] <- NA
        data_spei_obs_mon_event[which(!is.na(data_spei_obs_mon_event))] <- 1
        
        # (Event 1) How many water scarcity event occured from 1990-2015? WEI+ > 20
        wei_index_seasonal_seas_event_20 <- wei_index_seasonal_seas
        wei_index_seasonal_seas_event_20[which(wei_index_seasonal_seas_event_20 < 20)] <- NA
        wei_index_seasonal_seas_event_20[which(!is.na(wei_index_seasonal_seas_event_20))] <- 1
        
        # (Event 2) How many water scarcity event occured from 1990-2015? WEI+ > 40
        wei_index_seasonal_seas_event_40 <- wei_index_seasonal_seas
        wei_index_seasonal_seas_event_40[which(wei_index_seasonal_seas_event_40 < 40)] <- NA
        wei_index_seasonal_seas_event_40[which(!is.na(wei_index_seasonal_seas_event_40))] <- 1        
        
        # How many severe drought events happened when WEI+ > 20? 
        data_spei_obs_mon_event_with_wei_20 <- data_spei_obs_mon_event
        data_spei_obs_mon_event_with_wei_20[which(is.na(wei_index_seasonal_seas_event_20))] <- NA
        
        # How many severe drought events happened when WEI+ > 40? 
        data_spei_obs_mon_event_with_wei_40 <- data_spei_obs_mon_event
        data_spei_obs_mon_event_with_wei_40[which(is.na(wei_index_seasonal_seas_event_40))] <- NA      

        
        ###############################################################
        ###############################################################
        ################## POPULATION EXPOSURE ########################
        ###############################################################
        ###############################################################
        
        # Total Population 
        population_region_seasonal_seas <- array_population 
        population_exposed_total <- apply(population_region_seasonal_seas, 1, sum)
        
        # Population exposed to severe drought SPEI < -1.3
        population_exposed_spei <- population_region_seasonal_seas
        population_exposed_spei[which(is.na(data_spei_obs_mon_event))] <- NA
        population_exposed_annual <- apply(population_exposed_spei, 1, sum, na.rm= T)
        ratio_spei_severe <- (population_exposed_annual/population_exposed_total)*100
        
        # Population exposed to WEI > 20
        
        population_exposed_wei_20 <- population_region_seasonal_seas
        population_exposed_wei_20[which(is.na(wei_index_seasonal_seas_event_20))] <- NA
        population_exposed_annual <- apply(population_exposed_wei_20, 1, sum, na.rm= T)
        ratio_wei_20 <- (population_exposed_annual/population_exposed_total)*100
        
        # Population exposed to WEI > 40
        
        population_exposed_wei_40 <- population_region_seasonal_seas
        population_exposed_wei_40[which(is.na(wei_index_seasonal_seas_event_40))] <- NA
        population_exposed_annual <- apply(population_exposed_wei_40, 1, sum, na.rm= T)
        ratio_wei_40 <- (population_exposed_annual/population_exposed_total)*100
        
        # Population exposed to WEI > 20 and SPEI < -1.3
        
        population_exposed_wei_20_spei_severe <- population_region_seasonal_seas
        population_exposed_wei_20_spei_severe[which(is.na(data_spei_obs_mon_event_with_wei_20))] <- NA
        population_exposed_annual <- apply(population_exposed_wei_20_spei_severe, 1, sum, na.rm= T)
        ratio_wei_20_spei_severe <- (population_exposed_annual/population_exposed_total)*100
        
        # Population exposed to WEI > 40 and SPEI < -1.3
        
        population_exposed_wei_40_spei_severe <- population_region_seasonal_seas
        population_exposed_wei_40_spei_severe[which(is.na(data_spei_obs_mon_event_with_wei_40))] <- NA
        population_exposed_annual <- apply(population_exposed_wei_40_spei_severe, 1, sum, na.rm= T)
        ratio_wei_40_spei_severe <- (population_exposed_annual/population_exposed_total)*100      
        
        # Smooth lines
        x_axis_values <- 1:30
        lo_spei_severe <- loess(ratio_spei_severe~x_axis_values)
        lo_wei_20 <-  loess(ratio_wei_20~x_axis_values)
        lo_wei_40 <-  loess(ratio_wei_40~x_axis_values)
        lo_wei_20_spei_severe <-  loess(ratio_wei_20_spei_severe~x_axis_values)
        lo_wei_40_spei_severe <-  loess(ratio_wei_40_spei_severe~x_axis_values)
        
        ratio_spei_severe_smoothed <- predict(lo_spei_severe)
        ratio_wei_20_smoothed <- predict(lo_wei_20)
        ratio_wei_40_smoothed <- predict(lo_wei_40)
        ratio_wei_20_spei_severe_smoothed <- predict(lo_wei_20_spei_severe)
        ratio_wei_40_spei_severe_smoothed <- predict(lo_wei_40_spei_severe)
        
        # Colors for each category
        col_spei_severe <- '#d94801'
        col_spei_severe_smooth <- '#7f2704'
        
        col_wei_20 <- "#c51b7d"
        col_wei_20_smooth <- "#c51b7d"
        
        col_wei_40 <- "#2171b5"
        col_wei_40_smooth <- '#6baed6'
        
        col_wei_20_spei_severe <- "#98c8d9"
        col_wei_20_spei_severe_smooth <- "#98c8d9"
        
        col_wei_40_spei_severe <- "black"       # "#525252" 
        col_wei_40_spei_severe_smooth <- "black" #"#969696" 
        
        # Plotting
        
        png(filename = paste(path.intermediate_data, 'figures/6_population_and_area_exposure_fin/WEI20/population_exposure_SPEI_',spei_index,'-',target_months_name,'_WEIplus_',seas_wei_name,'_20.png', sep=""),width=1000, height=800, res = 150)
        
        par(xpd=FALSE)
        plot(x_axis_values,ratio_spei_severe,xlim = c(1,n_year),ylim = c(0,48), type = 'l', lty=2, col=col_spei_severe, lwd=1.2, xaxt="n", ylab="Percentage", xlab="", main = " ")
        axis(1, at=seq(1,30,by=1), label=1990:2019)
        lines(x_axis_values, predict(lo_spei_severe), col=col_spei_severe, lwd=2)
        
        lines(x_axis_values, ratio_wei_20, type = 'l', lty=2, col=col_wei_20, lwd=1.5)
        lines(x_axis_values, predict(lo_wei_20), col=col_wei_20, lwd=2)
        
        # lines(x_axis_values, ratio_wei_40, type = 'l', lty=2, col=col_wei_40, lwd=1.2)
        # lines(x_axis_values, predict(lo_wei_40), col=col_wei_40, lwd=2)
        
        lines(x_axis_values, ratio_wei_20_spei_severe, type = 'l', lty=2, col=col_wei_20_spei_severe, lwd=1.5)
        lines(x_axis_values, predict(lo_wei_20_spei_severe), col=col_wei_20_spei_severe, lwd=2)
        
        # lines(x_axis_values, ratio_wei_40_spei_severe, type = 'l', lty=2, col=col_wei_40_spei_severe, lwd=1.2)
        # lines(x_axis_values, predict(lo_wei_40_spei_severe), col=col_wei_40_spei_severe, lwd=2)
        
        par(xpd=TRUE)
        # legend("topright", inset=c(-0.2,0), legend=c("A","B"), pch=c(1,3), title="Group")
        legend(1,-5.8, legend = c("Population exposed to severe drought conditions (SPEI12 < -1.3)", "Population exposed to severe water stress conditions (WEI+ > 20)", "Population exposed to water stress during severe drought conditions"),
               col = c(col_spei_severe, col_wei_20, col_wei_20_spei_severe), lwd = 2,
               title = "", horiz = FALSE, cex = 0.8, box.lwd = 0, bg = "transparent")
        
        dev.off()
        
        # Save data
        
        # combined_df <- cbind(Year,ratio_spei_severe,ratio_spei_severe_smoothed,ratio_wei_20,ratio_wei_20_smoothed,ratio_wei_40,ratio_wei_40_smoothed,
        #                      ratio_wei_20_spei_severe,ratio_wei_20_spei_severe_smoothed,ratio_wei_40_spei_severe,ratio_wei_40_spei_severe_smoothed)
        # 
        # colnames(combined_df) <- c("Year","Percentage_spei_severe","Percentage_spei_severe_smoothed","Percentage_wei_20","Percentage_wei_20_smoothed","Percentage_wei_40","Percentage_wei_40_smoothed",
        #                            "Percentage_wei_20_spei_severe","Percentage_wei_20_spei_severe_smoothed","Percentage_wei_40_spei_severe","Percentage_wei_40_spei_severe_smoothed")
        # 
        # write.table(combined_df, file = paste(path.final_data, 'population_exposure_SPEI_',spei_index,'-',target_months_name,'_WEIplus_',seas_wei_name,'.csv', sep=""), sep = ",", col.names = NA, qmethod = "double")
        
        ###############################################################
        ###############################################################
        ##################### AREA EXPOSURE ###########################
        ###############################################################
        ###############################################################
    
        # Total Population 
        population_region_seasonal_seas <- array_area 
        population_exposed_total <- apply(population_region_seasonal_seas, 1, sum)
        
        # Population exposed to severe drought SPEI < -1.3
        population_exposed_spei <- population_region_seasonal_seas
        population_exposed_spei[which(is.na(data_spei_obs_mon_event))] <- NA
        population_exposed_annual <- apply(population_exposed_spei, 1, sum, na.rm= T)
        ratio_spei_severe <- (population_exposed_annual/population_exposed_total)*100
        
        # Population exposed to WEI > 20
        
        population_exposed_wei_20 <- population_region_seasonal_seas
        population_exposed_wei_20[which(is.na(wei_index_seasonal_seas_event_20))] <- NA
        population_exposed_annual <- apply(population_exposed_wei_20, 1, sum, na.rm= T)
        ratio_wei_20 <- (population_exposed_annual/population_exposed_total)*100
        
        # Population exposed to WEI > 40
        
        population_exposed_wei_40 <- population_region_seasonal_seas
        population_exposed_wei_40[which(is.na(wei_index_seasonal_seas_event_40))] <- NA
        population_exposed_annual <- apply(population_exposed_wei_40, 1, sum, na.rm= T)
        ratio_wei_40 <- (population_exposed_annual/population_exposed_total)*100
        
        # Population exposed to WEI > 20 and SPEI < -1.3
        
        population_exposed_wei_20_spei_severe <- population_region_seasonal_seas
        population_exposed_wei_20_spei_severe[which(is.na(data_spei_obs_mon_event_with_wei_20))] <- NA
        population_exposed_annual <- apply(population_exposed_wei_20_spei_severe, 1, sum, na.rm= T)
        ratio_wei_20_spei_severe <- (population_exposed_annual/population_exposed_total)*100
        
        # Population exposed to WEI > 40 and SPEI < -1.3
        
        population_exposed_wei_40_spei_severe <- population_region_seasonal_seas
        population_exposed_wei_40_spei_severe[which(is.na(data_spei_obs_mon_event_with_wei_40))] <- NA
        population_exposed_annual <- apply(population_exposed_wei_40_spei_severe, 1, sum, na.rm= T)
        ratio_wei_40_spei_severe <- (population_exposed_annual/population_exposed_total)*100      
        
        # Smooth lines
        x_axis_values <- 1:30
        lo_spei_severe <- loess(ratio_spei_severe~x_axis_values)
        lo_wei_20 <-  loess(ratio_wei_20~x_axis_values)
        lo_wei_40 <-  loess(ratio_wei_40~x_axis_values)
        lo_wei_20_spei_severe <-  loess(ratio_wei_20_spei_severe~x_axis_values)
        lo_wei_40_spei_severe <-  loess(ratio_wei_40_spei_severe~x_axis_values)
        
        ratio_spei_severe_smoothed <- predict(lo_spei_severe)
        ratio_wei_20_smoothed <- predict(lo_wei_20)
        ratio_wei_40_smoothed <- predict(lo_wei_40)
        ratio_wei_20_spei_severe_smoothed <- predict(lo_wei_20_spei_severe)
        ratio_wei_40_spei_severe_smoothed <- predict(lo_wei_40_spei_severe)
        
        # Colors for each category
        col_spei_severe <- '#d94801'
        col_spei_severe_smooth <- '#7f2704'
        
        col_wei_20 <- "#c51b7d"
        col_wei_20_smooth <- "#c51b7d"
        
        col_wei_40 <- "#2171b5"
        col_wei_40_smooth <- '#6baed6'
        
        col_wei_20_spei_severe <- "#98c8d9"
        col_wei_20_spei_severe_smooth <- "#98c8d9"
        
        col_wei_40_spei_severe <- "black"       # "#525252" 
        col_wei_40_spei_severe_smooth <- "black" #"#969696" 
        
        # Plotting
        
        png(filename = paste(path.intermediate_data, 'figures/6_population_and_area_exposure_fin/WEI20/area_exposure_SPEI_',spei_index,'-',target_months_name,'_WEIplus_',seas_wei_name,'_20.png', sep=""),width=1000, height=800, res = 150)
        
        par(xpd=FALSE)
        plot(x_axis_values,ratio_spei_severe,xlim = c(1,n_year),ylim = c(0,45), type = 'l', lty=2, col=col_spei_severe, lwd=1.2, xaxt="n", ylab="Percentage", xlab="", main = " ")
        axis(1, at=seq(1,30,by=1), label=1990:2019)
        lines(x_axis_values, predict(lo_spei_severe), col=col_spei_severe, lwd=2)
        
        lines(x_axis_values, ratio_wei_20, type = 'l', lty=2, col=col_wei_20, lwd=1.5)
        lines(x_axis_values, predict(lo_wei_20), col=col_wei_20, lwd=2)
        
        # lines(x_axis_values, ratio_wei_40, type = 'l', lty=2, col=col_wei_40, lwd=1.2)
        # lines(x_axis_values, predict(lo_wei_40), col=col_wei_40, lwd=2)
        
        lines(x_axis_values, ratio_wei_20_spei_severe, type = 'l', lty=2, col=col_wei_20_spei_severe, lwd=1.5)
        lines(x_axis_values, predict(lo_wei_20_spei_severe), col=col_wei_20_spei_severe, lwd=2)
        
        # lines(x_axis_values, ratio_wei_40_spei_severe, type = 'l', lty=2, col=col_wei_40_spei_severe, lwd=1.2)
        # lines(x_axis_values, predict(lo_wei_40_spei_severe), col=col_wei_40_spei_severe, lwd=2)
        
        par(xpd=TRUE)
        # legend("topright", inset=c(-0.2,0), legend=c("A","B"), pch=c(1,3), title="Group")
        legend(1,-5.8, legend = c("Area exposed to severe drought conditions (SPEI12 < -1.3)", "Area exposed to water stress conditions (WEI+ > 20)", "Area exposed to water stress during severe drought conditions"),
               col = c(col_spei_severe, col_wei_20, col_wei_20_spei_severe), lwd = 2,
               title = "", horiz = FALSE, cex = 0.8, box.lwd = 0, bg = "transparent")
        
        dev.off()
        
        # Save data
        
        # combined_df <- cbind(Year,ratio_spei_severe,ratio_spei_severe_smoothed,ratio_wei_20,ratio_wei_20_smoothed,ratio_wei_40,ratio_wei_40_smoothed,
        #                      ratio_wei_20_spei_severe,ratio_wei_20_spei_severe_smoothed,ratio_wei_40_spei_severe,ratio_wei_40_spei_severe_smoothed)
        # 
        # colnames(combined_df) <- c("Year","Percentage_spei_severe","Percentage_spei_severe_smoothed","Percentage_wei_20","Percentage_wei_20_smoothed","Percentage_wei_40","Percentage_wei_40_smoothed",
        #                            "Percentage_wei_20_spei_severe","Percentage_wei_20_spei_severe_smoothed","Percentage_wei_40_spei_severe","Percentage_wei_40_spei_severe_smoothed")
        # 
        # write.table(combined_df, file = paste(path.final_data, 'area_exposure_SPEI_',spei_index,'-',target_months_name,'_WEIplus_',seas_wei_name,'.csv', sep=""), sep = ",", col.names = NA, qmethod = "double")
        # 
    
  } # seas
  
} # spei - i
  



